searchData={"items":[{"type":"module","title":"CozyKV","doc":"Validates data containing key-value pairs.","ref":"CozyKV.html"},{"type":"module","title":"Features - CozyKV","doc":"* Supports any type of key, not limited to atoms.\n  * Provides errors with detailed metadata, which is useful to build custom error messages.","ref":"CozyKV.html#module-features"},{"type":"module","title":"Terminologies - CozyKV","doc":"* spec - a list which describes the structure of key-value pairs.\n  * data - the key-value pairs to be validated using a spec.","ref":"CozyKV.html#module-terminologies"},{"type":"module","title":"Usage - CozyKV","doc":"```elixir\n# 1. create a raw spec\nspec = [\n  name: [type: :string, required: true]\n]\n\n# 2. validate the spec\nspec = CozyKV.validate_spec!(spec)\n\n# 3. validate a piece of data using above spec\n\n# 3.1 validate key-value pairs in list form.\nkv_list = [name: \"zeke\"]\n{:ok, [name: \"zeke\"]} = CozyKV.validate(spec, kv_list)\n\n# 3.2 validate key-value pairs in map form.\nkv_map = %{name: \"zeke\"}\n{:ok, %{name: \"zeke\"}} = CozyKV.validate(spec, kv_map)\n\n# 3.3 when validation fails, an error is returned\nkv_bad = []\n{:error, %CozyKV.ValidationError{path: _, type: _}} = CozyKV.validate(spec, kv_bad)\n```","ref":"CozyKV.html#module-usage"},{"type":"module","title":"Specs - CozyKV","doc":"A valid spec is a list of two-element tuples.\n\nIn each tuple, the first element is the key, the second element is the spec\noptions of the value correspoding to the key.\n\n<!-- tabs-open -->","ref":"CozyKV.html#module-specs"},{"type":"module","title":"atom key - CozyKV","doc":"```elixir\n[\n  name: [\n    type: :string,\n    required: true\n  ],\n  # ...\n]\n```","ref":"CozyKV.html#module-atom-key"},{"type":"module","title":"string key - CozyKV","doc":"```elixir\n[\n  {\"name\", [\n    type: :string,\n    required: true\n  ]},\n  # ...\n]\n```","ref":"CozyKV.html#module-string-key"},{"type":"module","title":"arbitrary key - CozyKV","doc":"```elixir\n# allows to use arbitrary key in any type.\n[\n  {%{}, [\n    type: :string,\n    required: true\n  ]},\n  # ...\n]\n```\n\n<!-- tabs-close -->","ref":"CozyKV.html#module-arbitrary-key"},{"type":"module","title":"Spec options - CozyKV","doc":"These options are supported:\n\n  * `:type`\n  * `:required`\n  * `:default`\n  * `:deprecated`\n  * `:doc`","ref":"CozyKV.html#module-spec-options"},{"type":"module","title":"Types and Values - CozyKV","doc":"> The left side of `-` is the type name, and the right side is the description\n> of the value corresponding to the type.\n\n  * `:any` - Any value.\n\n  * `nil` - `nil` itself.\n\n  * `:atom` - An atom.\n\n  * `:boolean` - A boolean.\n\n  * `:string` - A string.\n\n  * `:integer` - An integer.\n\n  * `:pos_integer` - A positive integer.\n\n  * `:non_neg_integer` - A non-negative integer.\n\n  * `:float` - A float.\n\n  * `:tuple` - A tuple.\n\n  * `{:tuple, subtypes}` - A tuple as described by `subtypes`. The length of\n    the expected tuple must match the length of `subtypes`. The type of each\n    element in tuple must match the type at the same position of `subtypes`.\n    For example, a valid value of `{:tuple, [:atom, :string, {:list, :integer}]}`\n    can be `{:name, \"zeke\", [3, 2, 2]}`.\n\n  * `:list` - A list.\n\n  * `{:list, subtype}` - A list with elements of type `subtype`.\n\n  * `:keyword_list` - A keyword list.\n\n  * `{:keyword_list, spec}` - A keyword list with key-value pairs structured\n    by `spec`.\n\n  * `:non_empty_keyword_list` - A non-empty keyword list.\n\n  * `{:non_empty_keyword_list, spec}` - A non-empty keyword list with key-value\n    pairs structured by `spec`.\n\n  * `:map` - A map with atom keys. It is a shortcut of `{:map, :atom, :any}`.\n\n  * `{:map, key_type, value_type}` - A map with `key_type` keys and `value_type` values.\n\n  * `{:map, spec}` - A map with key-value pairs structured by `spec`.\n\n  * `{:struct, struct_name}` - A struct.\n\n  * `:timeout` - A non-negative integer or the atom `:infinity`.\n\n  * `:pid` - A PID.\n\n  * `:reference` - A reference (see `t:reference/0`).\n\n  * `:mfa` - A tuple in the format `{mod, fun, args}`.\n\n  * `{:fun, arity}` - A function with a specific `arity`.\n\n  * `:mod_args` - A tuple in the format `{mod, args}`. It is usually used\n    for process initialization using `start_link` and similar.\n\n  * `{:value_in, values}` - A value which is one of the values in `values`.\n    `values` can be any enumerable value, such as a list or a `%Range{}`.\n\n  * `{:type_in, subtypes}` - A value which matches one of the types in `subtypes`.\n    `subtypes` is a list of types.\n\n  * `{:custom, mod, fun}` - A custom type. The related value will be validated\n    by `apply(mod, fun, [type, value, metadata])`.\n    `type` is the `{:custom, mod, fun}` itself. `mod.fun(type, value, metadata)`\n    must return `{:ok, value}` or `{:error, exception}`.","ref":"CozyKV.html#module-types-and-values"},{"type":"module","title":"Data - CozyKV","doc":"A valid piece of data is a list of two-element tuples or a map.\n\n<!-- tabs-open -->","ref":"CozyKV.html#module-data"},{"type":"module","title":"list (atom key) - CozyKV","doc":"```elixir\n[\n  name: \"zeke\",\n  # ...\n]\n```","ref":"CozyKV.html#module-list-atom-key"},{"type":"module","title":"list (string key) - CozyKV","doc":"```elixir\n[\n  {\"name\", \"zeke\"},\n  # ...\n]\n```","ref":"CozyKV.html#module-list-string-key"},{"type":"module","title":"list (arbitrary key) - CozyKV","doc":"```elixir\n# In general, you don't do this.\n# I just want to demonstrate what this package can do.\n[\n  {%{}, \"zeke\"},\n  # ...\n]\n```\n\n<!-- tabs-close -->\n\n<!-- tabs-open -->","ref":"CozyKV.html#module-list-arbitrary-key"},{"type":"module","title":"map (atom key) - CozyKV","doc":"```elixir\n%{\n  name: \"zeke\",\n  # ...\n}\n```","ref":"CozyKV.html#module-map-atom-key"},{"type":"module","title":"map (string key) - CozyKV","doc":"```elixir\n%{\n  \"name\" => \"zeke\",\n  # ...\n}\n```","ref":"CozyKV.html#module-map-string-key"},{"type":"module","title":"map (arbitrary key) - CozyKV","doc":"```elixir\n# In general, you don't do this.\n# I just want to demonstrate what this package can do.\n%{\n  %{} => \"zeke\",\n  # ...\n}\n```\n\n<!-- tabs-close -->","ref":"CozyKV.html#module-map-arbitrary-key"},{"type":"module","title":"Use cases - CozyKV","doc":"* validate options (like what [nimble_options](https://hex.pm/packages/nimble_options) does).\n  * validate maps parsed from a JSON or YAML files.","ref":"CozyKV.html#module-use-cases"},{"type":"module","title":"Limitations - CozyKV","doc":"* `validate_spec!/1` doesn't validate the inner spec of a nested spec.","ref":"CozyKV.html#module-limitations"},{"type":"module","title":"Todo - CozyKV","doc":"* doc generation","ref":"CozyKV.html#module-todo"},{"type":"function","title":"CozyKV.validate/2","doc":"Validates a key-value data.","ref":"CozyKV.html#validate/2"},{"type":"function","title":"CozyKV.validate!/2","doc":"Bang version of `validate/2`.","ref":"CozyKV.html#validate!/2"},{"type":"function","title":"CozyKV.validate_spec!/1","doc":"Validates a spec.","ref":"CozyKV.html#validate_spec!/1"},{"type":"type","title":"CozyKV.data/0","doc":"","ref":"CozyKV.html#t:data/0"},{"type":"type","title":"CozyKV.spec/0","doc":"","ref":"CozyKV.html#t:spec/0"},{"type":"exception","title":"CozyKV.ValidationError","doc":"The error that is returned (or raised) when there is an issue in the\nvalidation process.","ref":"CozyKV.ValidationError.html"},{"type":"exception","title":"Error messages - CozyKV.ValidationError","doc":"By default, the error struct doesn't include message.\n\nIf you want to turn an error into a human-readable message, you should\nuse `Exception.message/1`.\n\nIf you don't like the default error message, you are free to customize your\nown version using the information provided by the error struct.","ref":"CozyKV.ValidationError.html#module-error-messages"},{"type":"type","title":"CozyKV.ValidationError.invalid_value/0","doc":"","ref":"CozyKV.ValidationError.html#t:invalid_value/0"},{"type":"type","title":"CozyKV.ValidationError.key/0","doc":"","ref":"CozyKV.ValidationError.html#t:key/0"},{"type":"type","title":"CozyKV.ValidationError.missing_key/0","doc":"","ref":"CozyKV.ValidationError.html#t:missing_key/0"},{"type":"type","title":"CozyKV.ValidationError.t/0","doc":"","ref":"CozyKV.ValidationError.html#t:t/0"},{"type":"type","title":"CozyKV.ValidationError.type/0","doc":"","ref":"CozyKV.ValidationError.html#t:type/0"},{"type":"type","title":"CozyKV.ValidationError.unknown_keys/0","doc":"","ref":"CozyKV.ValidationError.html#t:unknown_keys/0"},{"type":"type","title":"CozyKV.ValidationError.value/0","doc":"","ref":"CozyKV.ValidationError.html#t:value/0"},{"type":"extras","title":"CozyKV","doc":"# CozyKV\n\n[![CI](https://github.com/cozy-elixir/cozy_kv/actions/workflows/ci.yml/badge.svg)](https://github.com/cozy-elixir/cozy_kv/actions/workflows/ci.yml)\n[![Hex.pm](https://img.shields.io/hexpm/v/cozy_kv.svg)](https://hex.pm/packages/cozy_kv)\n\nValidates data containing key-value pairs.","ref":"readme.html"},{"type":"extras","title":"Installation - CozyKV","doc":"Add `:cozy_kv` to the list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:cozy_kv,  }\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - CozyKV","doc":"For more information, see the [documentation](https://hexdocs.pm/cozy_kv).","ref":"readme.html#usage"},{"type":"extras","title":"License - CozyKV","doc":"[Apache License 2.0](https://www.apache.org/licenses/LICENSE-2.0)","ref":"readme.html#license"},{"type":"extras","title":"About Layers","doc":"# About Layers\n\n> Describes how to build this package layer by layer.","ref":"about-layers.html"},{"type":"extras","title":"Layer 1 - primitives - About Layers","doc":"Primitives provide basic support for describing a key-value pair.","ref":"about-layers.html#layer-1-primitives"},{"type":"extras","title":"Key names - About Layers","doc":"A key name can be:\n\n- any type of data, such as atoms, strings, integers, maps, lists, etc.\n- a special one - `:*`, which means several keys that have same set of key attributes.","ref":"about-layers.html#key-names"},{"type":"extras","title":"Key attributes - About Layers","doc":"Key attributes describe the key and value.\n\nAvailable attributes are:\n\n- `:type` - specify the type of value.\n- `:required` - specify whether the key is required.\n- `:default` - specify the default value of the key.\n- `:doc` - specify the doc of the key.\n- ...","ref":"about-layers.html#key-attributes"},{"type":"extras","title":"Layer 2 - meta specs - About Layers","doc":"A meta spec is the spec of a valid spec.\n\nA meta spec is also key-value pairs, so we can describe it with primitives.\n\nFor example:\n\n```text\n[\n  *: [\n    type:\n      {:keyword_list,\n        [\n          type: [\n            type: {:custom, Type, :validate_type},\n            default: :any,\n            doc: \"...\"\n          ],\n          required: [\n            type: :boolean,\n            default: false,\n            doc: \"...\"\n          ],\n          default: [\n            type: :any,\n            doc: \"...\"\n          ],\n          doc: [\n            type: {:type_in, [:string, {:in, [false]}]},\n            doc: \"...\"\n          ]\n        ]}\n  ]\n]\n```\n\nAbove meta spec describes the structure of a valid spec:\n\n- It's a keyword list.\n- The key can be any type of data.\n- The value should be a keyword list, where following keys are expected.\n  - :type\n  - :required\n  - :default\n  - :doc","ref":"about-layers.html#layer-2-meta-specs"},{"type":"extras","title":"Layer 3 - specs - About Layers","doc":"A spec is for validating data.\n\nIt conforms the meta spec above.\n\nFor example:\n\n```text\n[\n  name: [\n    type: :string,\n    required: true,\n    doc: \"The name of server\"\n  ],\n  enabled?: [\n    type: :boolean,\n    required: false,\n    default: true,\n    doc: \"Whether the server is enabled.\"\n  ],\n]\n```","ref":"about-layers.html#layer-3-specs"},{"type":"extras","title":"Layer 4 - validator - About Layers","doc":"Validator is for validating that:\n\n- spec follows meta spec.\n- data follows spec.\n\nThe chain of validation is:\n\n```text\n   [meta_spec]\n        |\n        |\n[spec] ---> validated spec\n     validate   |\n                |\n        [data] ---> validated data\n             validate\n```","ref":"about-layers.html#layer-4-validator"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}